*Program to Estimate Multiple Spatial Lag Models 
*For ALM replication
*Written by Jude Hays 4/2009
*Revised by Aya K. 6/2009 (replication of the PolNet resul, Stata10 adjustment)


clear
pr drop _all
set more off
set matsize 368

***********************
*Likelihood Evaluator  
***********************                                                         

program define splag_ll

args lnf mu rho1 rho2 rho3 rho4 sigma
tempvar A rSL1 rSL2 rSL3 rSL4
gen double `rSL1'=`rho1'*SL1
gen double `rSL2'=`rho2'*SL2
gen double `rSL3'=`rho3'*SL3
gen double `rSL4'=`rho4'*SL4
scalar p1 = `rho1'
scalar p2 = `rho2'
scalar p3 = `rho3'
scalar p4 = `rho4'
matrix p1W1 = p1*W1
matrix p2W2 = p2*W2
matrix p3W3 = p3*W3
matrix p4W4 = p4*W4
matrix IpW = I_n - p1W1 - p2W2- p3W3- p4W4
qui gen double `A' = ln(det(IpW))/$nobs if _n == 1
scalar A = `A'
*qui replace `lnf'= A + ln(normden($ML_y1-`rSL1'-`rSL2'-`rSL3'-`rSL4'-`mu', 0, `sigma'))
* For Stata10: use "normalden" instead of "normden"
qui replace `lnf'= A + ln(normalden($ML_y1-`rSL1'-`rSL2'-`rSL3'-`rSL4'-`mu', 0, `sigma'))
end


**********************
*Open Data For Weights
**********************

use "C:\Documents and Settings\Administrator\Desktop\mstar\border_ver9.dta", clear
qui sum var1
global nobs = r(N)
mkmat var1-var$nobs, matrix(W1)
matrix I_n = I($nobs)

drop _all
use "C:\Documents and Settings\Administrator\Desktop\mstar\eu_ver9.dta", clear
mkmat var1-var$nobs, matrix(W2)

drop _all
use "C:\Documents and Settings\Administrator\Desktop\mstar\trade_ver9.dta", clear
mkmat var1-var$nobs, matrix(W3)

drop _all
use "C:\Documents and Settings\Administrator\Desktop\mstar\poldistance_rsver9.dta", clear
mkmat var1-var$nobs, matrix(W4)

**************************
*Open Data for Regression
**************************
drop _all
use "C:\Documents and Settings\Administrator\Desktop\mstar\data_alm_ver9.dta", clear

global Y real_alm_spending
global X temp_lag real_growth sur union deind trade work_age leftc tcdemc llvote c2-c21 y2-y22
mkmat $Y, matrix(Y)
matrix SL1 = W1*Y
svmat SL1, n(SL1)
matrix SL2 = W2*Y
svmat SL2, n(SL2)
matrix SL3 = W3*Y
svmat SL3, n(SL3)
matrix SL4 = W4*Y
svmat SL4, n(SL4)

************************
*Produce starting values
************************ 

qui regress $Y $X
matrix OLSb=e(b)
local OLSsigma=e(rmse)

***************************
*Estimate spatial lag model
***************************

ml model lf splag_ll (mu: $Y=$X) (rho1:) (rho2:) (rho3:) (rho4:) (sigma:)
ml init OLSb
ml init rho1:_cons=0
ml init rho2:_cons=0
ml init rho3:_cons=0
ml init rho4:_cons=0
ml init sigma:_cons=`OLSsigma'
*ml trace on
ml max

	
